#!/usr/bin/env python
import random
import pygame
import main

PLAYER_1_START_TAG = (255, 0, 0, 255)
PLAYER_1_DWARF_PLACE_TAG = (255, 64, 64, 255)
PLAYER_1_FLOWER_PLACE_TAG = (255, 128, 128, 255)

PLAYER_2_START_TAG = (0, 255, 0, 255)
PLAYER_2_DWARF_PLACE_TAG = (64, 255, 64, 255)
PLAYER_2_FLOWER_PLACE_TAG = (128, 255, 128, 255)

PLAYER_3_START_TAG = (0, 0, 255, 255)
PLAYER_3_DWARF_PLACE_TAG = (64, 64, 255, 255)
PLAYER_3_FLOWER_PLACE_TAG = (128, 128, 255, 255)

PLAYER_4_START_TAG = (255, 255, 0, 255)
PLAYER_4_DWARF_PLACE_TAG = (255, 255, 64, 255)
PLAYER_4_FLOWER_PLACE_TAG = (255, 255, 128, 255)

class PlayerInfo:
    def __init__( self, start, dwarf_place, flower_place ):
        self.start_tag = start
        self.dwarf_place_tag = dwarf_place
        self.flower_place_tag = flower_place
        return

player_1 = PlayerInfo( PLAYER_1_START_TAG, PLAYER_1_DWARF_PLACE_TAG, PLAYER_1_FLOWER_PLACE_TAG )
player_2 = PlayerInfo( PLAYER_2_START_TAG, PLAYER_2_DWARF_PLACE_TAG, PLAYER_2_FLOWER_PLACE_TAG )
player_3 = PlayerInfo( PLAYER_3_START_TAG, PLAYER_3_DWARF_PLACE_TAG, PLAYER_3_FLOWER_PLACE_TAG )
player_4 = PlayerInfo( PLAYER_4_START_TAG, PLAYER_4_DWARF_PLACE_TAG, PLAYER_4_FLOWER_PLACE_TAG )
players = [ player_1, player_2, player_3, player_4 ]

JUST_WALKING = 0
TRY_TO_TAKE_ITEM = 1
PUT_ITEM_DOWN = 2

TASK_INITIALIZE_GARDENER = 0
TASK_RUNNING = 1

class Gardener:
    def __init__( self, id, human, own_surface, speed, ai, sfx, size ):
        self.id = id
        self.human = human
        self.playerinfo = players[id]
        self.surface = pygame.transform.scale( own_surface, (size[0],size[1]) )

        self.speed = speed
        self.ai = ai
        self.ai.add_player( self )
        
        self.sfx = sfx

        self.width = size[0]
        self.height = size[1]
        self.center = None

        self.path = []
        self.state = JUST_WALKING
        self.item = None
        self.task = TASK_INITIALIZE_GARDENER

        return

    def draw( self, surface ):
        if self.center == None:
            self.center = self.ai.get_start_position( players[self.id] )
        upper_left = ( self.center[0]-self.width/2, self.center[1]-self.height/2 )
        surface.blit( self.surface, upper_left )
        
        points = []
        points.append( self.center )
        for point in self.path:
            points.append( point )
        #if points.__len__()>1:
        #    pygame.draw.lines( surface, (255,255,255,255), False, points, 3 )
        return

    def handle_keyboard( self, keys ):
        if self.center == None:
            self.center = self.ai.get_start_position( players[self.id] )
        
        diff_x = 0
        diff_y = 0
        
        speed = self.speed
        if self.item is not None:
            speed = self.item.speed()

        if keys[pygame.K_LEFT]:
            diff_x = -speed
        if keys[pygame.K_RIGHT]:
            diff_x = speed
        if keys[pygame.K_UP]:
            diff_y = -speed
        if keys[pygame.K_DOWN]:
            diff_y = speed
        
        new_center = ( self.center[0]+diff_x, self.center[1]+diff_y )
        if self.ai.is_point_passable( new_center ):
            self.center = new_center
        else:
            new_center = ( self.center[0]+diff_x, self.center[1] )
            if self.ai.is_point_passable( new_center ):
                self.center = new_center
            new_center = ( self.center[0], self.center[1]+diff_y )
            if self.ai.is_point_passable( new_center ):
                self.center = new_center
        
        if keys[pygame.K_SPACE]:
            if self.state == JUST_WALKING:
                if self.item == None:
                    self.state = TRY_TO_TAKE_ITEM
                    self.item = self.ai.take_item( self.center )
                    if self.item is not None:
                        print "take item at", self.center
                    else:
                        self.ai.get_off_my_lawn( self )
                        yells = ["get_off_01.wav","get_off_02.wav","get_off_03.wav"]
                        self.sfx.play_sound( yells[random.randint(0,yells.__len__()-1)] )
                else:
                    self.state = PUT_ITEM_DOWN
                    self.item = None
                    print "put item down at", self.center
        else:
            self.state = JUST_WALKING
        
        
        
        
        return

    def tick( self ):
        if self.center == None:
            self.center = self.ai.get_start_position( self.playerinfo )
            
        speed = self.speed
        if self.item is not None:
            speed = self.item.speed()

        # is this gardener controller by the ai?
        if self.human == False:
            if self.task == TASK_INITIALIZE_GARDENER:
                self.ai.assign_new_task( self )
                self.task = TASK_RUNNING
            
            # move along the path
            if self.path.__len__()>0:
                move_x = 0
                move_y = 0
                diff_x = self.path[0][0] - self.center[0]
                diff_y = self.path[0][1] - self.center[1]
                if diff_x != 0:
                    move_x = max(-speed,diff_x) if diff_x<0 else min(speed,diff_x)
                if diff_y != 0:
                    move_y = max(-speed,diff_y) if diff_y<0 else min(speed,diff_y)
                if int(move_x) == 0 and int(move_y) == 0:
                    self.path.pop(0)
                else:
                    self.center = ( self.center[0] + move_x, self.center[1] + move_y )
            else:
                #reached the destination so decide what to do now
                
                # does the gardener carry something?
                if self.item is not None:
                    # put item down and get a new task
                    self.item = None
                    self.ai.assign_new_task( self )
                    pass
                else:
                    # try to take an item from the ground
                    self.item = self.ai.take_item( self.center )
                    if self.item is not None:
                        # bring item back into the own garden
                        if self.item.name == "Dwarf":
                            self.ai.assign_path_to_home_dwarf_place( self )
                        else:
                            self.ai.assign_path_to_home_flower_place( self )
                    else:
                        # get new task
                        self.ai.assign_new_task( self )
                #self.path.append( self.ai.get_random_opponent_dwarf_position(self.playerinfo) )
        
        # update the position of the carried item        
        if self.item is not None:
            self.item.center = self.center

