import data
import pygame

import button
import gardener
import soundmanager
import aimanager

# Will hold the image surfaces
gfx = {}

# Global game settings
GAME_TITLE = "Get Off My Lawn, by Trobadour for PyWeek 08, Day 7"
SCREEN_SIZE = ( 800, 600 )
FRAMES_PER_SECOND = 60
BUTTON_COLOR_ENABLED = (0,0,0)
BUTTON_COLOR_DISABLED = (125,125,125)

# Graphic files
GFX_MAIN_MENU = "main.png"
GFX_HELP_SCREEN = "help.png"
GFX_BUTTON = "button_green.png"

GFX_LEVEL_01 = "level01_garden.png"
GFX_LEVEL_01_OBSTACLES = "level01_obstacles.png"
GFX_LEVEL_01_PLACES = "level01_places.png"
GFX_LEVEL_01_ITEMS = "level01_items.png"
GFX_LEVEL_01_OWNERS = "level01_owners.png"

GFX_LEVEL_02 = "level02_garden.png"
GFX_LEVEL_02_OBSTACLES = "level02_obstacles.png"
GFX_LEVEL_02_PLACES = "level02_places.png"
GFX_LEVEL_02_ITEMS = "level02_items.png"
GFX_LEVEL_02_OWNERS = "level02_owners.png"

GFX_LEVEL_03 = "level03_garden.png"
GFX_LEVEL_03_OBSTACLES = "level03_obstacles.png"
GFX_LEVEL_03_PLACES = "level03_places.png"
GFX_LEVEL_03_ITEMS = "level03_items.png"
GFX_LEVEL_03_OWNERS = "level03_owners.png"

GFX_LEVEL_04 = "level04_garden.png"
GFX_LEVEL_04_OBSTACLES = "level04_obstacles.png"
GFX_LEVEL_04_PLACES = "level04_places.png"
GFX_LEVEL_04_ITEMS = "level04_items.png"
GFX_LEVEL_04_OWNERS = "level04_owners.png"

GFX_GARDENDER_01 = "gardener01.png"
GFX_GARDENDER_02 = "gardener02.png"
GFX_GARDENDER_03 = "gardener03.png"
GFX_GARDENDER_04 = "gardener04.png"
GFX_DWARF = "dwarf.png"
GFX_FLOWER = "rose.png"

# Game modes
GAME_MODE_MAIN_MENU = 0
GAME_MODE_PLAYING = 1
GAME_MODE_HELP = 2

# Sound files and settings
SFX_BUFFER_LENGTH = 512
SFX_MUSIC = "country-1415.ogg"
SFX_BUTTON_CLICK = "button_click.wav"
SFX_GET_OFF_01 = "get_off_01.wav"
SFX_GET_OFF_02 = "get_off_02.wav"
SFX_GET_OFF_03 = "get_off_03.wav"

# Set mixer settings
pygame.mixer.pre_init( 44100, 16, 2, SFX_BUFFER_LENGTH )

def load_image( file, new_size=(0,0) ):
  filename = data.filepath( file )
  try:
      surface = pygame.image.load( filename )
  except pygame.error:
      raise SystemExit, 'Could not load image "%s" %s'%( filename, pygame.get_error())
  gfx[file] = surface.convert_alpha()
  if new_size != (0,0):
    gfx[file] = pygame.transform.scale( gfx[file], new_size )
  return

class Game:
    def __init__( self ):
        pygame.init()
        pygame.display.set_caption( GAME_TITLE )
        self.screen = self.set_fullscreen_mode( False )

        self.load_resources()

        self.ai = aimanager.AiManager( gfx[GFX_DWARF], gfx[GFX_FLOWER] )

        self.large_font = pygame.font.Font( None, 35 )
        self.small_font = pygame.font.Font( None, 30 )

        self.buttons = []
        self.game_mode = GAME_MODE_MAIN_MENU

        self.player = None
        self.opponents = None
        self.garden = None
        
        self.sfx.play_music( data.filepath(SFX_MUSIC) )

        return

    def load_resources( self ):
        self.sfx = soundmanager.SoundManager()
        self.sfx.load_sounds( [SFX_BUTTON_CLICK,SFX_GET_OFF_01,SFX_GET_OFF_02,SFX_GET_OFF_03] )
        load_image( GFX_MAIN_MENU, SCREEN_SIZE )
        load_image( GFX_HELP_SCREEN )
        load_image( GFX_BUTTON )
        load_image( GFX_LEVEL_01 )
        load_image( GFX_LEVEL_01_OBSTACLES )
        load_image( GFX_LEVEL_01_PLACES )
        load_image( GFX_LEVEL_01_ITEMS )
        load_image( GFX_LEVEL_01_OWNERS )
        load_image( GFX_LEVEL_02 )
        load_image( GFX_LEVEL_02_OBSTACLES )
        load_image( GFX_LEVEL_02_PLACES )
        load_image( GFX_LEVEL_02_ITEMS )
        load_image( GFX_LEVEL_02_OWNERS )
        load_image( GFX_LEVEL_03 )
        load_image( GFX_LEVEL_03_OBSTACLES )
        load_image( GFX_LEVEL_03_PLACES )
        load_image( GFX_LEVEL_03_ITEMS )
        load_image( GFX_LEVEL_03_OWNERS )
        load_image( GFX_LEVEL_04 )
        load_image( GFX_LEVEL_04_OBSTACLES )
        load_image( GFX_LEVEL_04_PLACES )
        load_image( GFX_LEVEL_04_ITEMS )
        load_image( GFX_LEVEL_04_OWNERS )
        load_image( GFX_GARDENDER_01 )
        load_image( GFX_GARDENDER_02 )
        load_image( GFX_GARDENDER_03 )
        load_image( GFX_GARDENDER_04 )
        load_image( GFX_DWARF )
        load_image( GFX_FLOWER )
        return

    def set_fullscreen_mode( self, fullscreen=True ):
        self.fullscreen = fullscreen
        if self.fullscreen:
            return pygame.display.set_mode( SCREEN_SIZE, pygame.FULLSCREEN )
        else:
            return pygame.display.set_mode( SCREEN_SIZE )

    def append_button( self, function, rect, text ):
        self.buttons.append( button.Button(function,rect,gfx[GFX_BUTTON],text,self.small_font,BUTTON_COLOR_ENABLED,BUTTON_COLOR_DISABLED) )
        return

    def change_game_mode( self, new_game_mode ):
        self.buttons = []
        self.game_mode = new_game_mode

        if new_game_mode == GAME_MODE_MAIN_MENU:
            self.append_button( self.on_button_play_level_01, (25,570,100,20), "Level 1" )
            self.append_button( self.on_button_play_level_02, (150,570,100,20), "Level 2" )
            self.append_button( self.on_button_play_level_03, (275,570,100,20), "Level 3" )
            self.append_button( self.on_button_play_level_04, (400,570,100,20), "Level 4" )
            self.append_button( self.on_button_fullscreen, (525,570,100,20), "Fullscreen" )
            self.append_button( self.on_button_help, (400,400,100,20), "Help" )
            self.append_button( self.on_button_quit, (650,570,100,20), "Quit" )
        if new_game_mode == GAME_MODE_PLAYING:
          self.append_button( self.on_button_main_menu, (600,530,150,20), "Main Menu" )
        if new_game_mode == GAME_MODE_HELP:
            self.append_button( self.on_button_main_menu, (600,530,150,20), "Main Menu" )
        return

    def on_button_quit( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.run_game = False
        return

    def on_button_play_level_01( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.garden = gfx[GFX_LEVEL_01]
        self.ai.initialize()
        self.player = gardener.Gardener( 0, True, gfx[GFX_GARDENDER_01], 2, self.ai, self.sfx, (30,60) )
        self.opponents = []
        self.ai.analyze( gfx[GFX_LEVEL_01_OBSTACLES], gfx[GFX_LEVEL_01_PLACES], gfx[GFX_LEVEL_01_ITEMS], gfx[GFX_LEVEL_01_OWNERS] )
        self.change_game_mode( GAME_MODE_PLAYING )
        return
    
    def on_button_play_level_02( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.garden = gfx[GFX_LEVEL_02]
        self.ai.initialize()
        self.player = gardener.Gardener( 0, True, gfx[GFX_GARDENDER_01], 2, self.ai, self.sfx, (30,60) )
        self.opponents = []
        self.opponents.append( gardener.Gardener(1,False,gfx[GFX_GARDENDER_02],2,self.ai,self.sfx,(30,60)) )
        self.ai.analyze( gfx[GFX_LEVEL_02_OBSTACLES], gfx[GFX_LEVEL_02_PLACES], gfx[GFX_LEVEL_02_ITEMS], gfx[GFX_LEVEL_02_OWNERS] )
        self.change_game_mode( GAME_MODE_PLAYING )
        return
    
    def on_button_play_level_03( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.garden = gfx[GFX_LEVEL_03]
        self.ai.initialize()
        self.player = gardener.Gardener( 0, True, gfx[GFX_GARDENDER_01], 2, self.ai, self.sfx, (30,60) )
        self.opponents = []
        self.opponents.append( gardener.Gardener(1,False,gfx[GFX_GARDENDER_02],2,self.ai,self.sfx,(30,60)) )
        self.opponents.append( gardener.Gardener(2,False,gfx[GFX_GARDENDER_03],2,self.ai,self.sfx,(30,60)) )
        self.ai.analyze( gfx[GFX_LEVEL_03_OBSTACLES], gfx[GFX_LEVEL_03_PLACES], gfx[GFX_LEVEL_03_ITEMS], gfx[GFX_LEVEL_03_OWNERS] )
        self.change_game_mode( GAME_MODE_PLAYING )
        return
    
    def on_button_play_level_04( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.garden = gfx[GFX_LEVEL_04]
        self.ai.initialize()
        self.player = gardener.Gardener( 0, True, gfx[GFX_GARDENDER_01], 2, self.ai, self.sfx, (30,60) )
        self.opponents = []
        self.opponents.append( gardener.Gardener(1,False,gfx[GFX_GARDENDER_02],2,self.ai,self.sfx,(30,60)) )
        self.opponents.append( gardener.Gardener(2,False,gfx[GFX_GARDENDER_03],2,self.ai,self.sfx,(30,60)) )
        self.opponents.append( gardener.Gardener(3,False,gfx[GFX_GARDENDER_04],2,self.ai,self.sfx,(30,60)) )
        self.ai.analyze( gfx[GFX_LEVEL_04_OBSTACLES], gfx[GFX_LEVEL_04_PLACES], gfx[GFX_LEVEL_04_ITEMS], gfx[GFX_LEVEL_04_OWNERS] )
        self.change_game_mode( GAME_MODE_PLAYING )
        return

    def on_button_main_menu( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.change_game_mode( GAME_MODE_MAIN_MENU )
        return
      
    def on_button_help( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.change_game_mode( GAME_MODE_HELP )
        return

    def on_button_help( self ):
        self.sfx.play_sound( SFX_BUTTON_CLICK )
        self.change_game_mode( GAME_MODE_HELP )
        return

    def on_button_fullscreen( self ):
        self.set_fullscreen_mode( not self.fullscreen )
        return

    def start( self ):
        self.run_game = True
        self.change_game_mode( GAME_MODE_MAIN_MENU )
        self.start_game_loop()
        return

    def handle_events( self ):
        if self.game_mode in [GAME_MODE_MAIN_MENU,GAME_MODE_HELP]:
            self.handle_events_menu()
        elif self.game_mode == GAME_MODE_PLAYING:
            self.handle_events_playing()
        return

    def handle_events_menu( self ):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.run_game = False
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                  if self.game_mode == GAME_MODE_HELP:
                    self.change_game_mode( GAME_MODE_MAIN_MENU )
                  else:
                    self.run_game = False
            if event.type == pygame.MOUSEBUTTONDOWN:
                mouse_pos = pygame.mouse.get_pos()
                for button in self.buttons:
                    button.handle_click_event( mouse_pos )
        return

    def handle_events_playing( self ):
      for event in pygame.event.get():

        if event.type == pygame.QUIT:
          self.run_game = False

        if event.type == pygame.KEYDOWN:
          if event.key == pygame.K_ESCAPE:
            self.change_game_mode( GAME_MODE_MAIN_MENU )

        if event.type == pygame.MOUSEBUTTONDOWN:
          mouse_pos = pygame.mouse.get_pos()
          for button in self.buttons:
            button.handle_click_event( mouse_pos )

      # get keyboard status
      keys = pygame.key.get_pressed()
      self.player.handle_keyboard( keys )
      return

    def tick( self ):
        if self.game_mode == GAME_MODE_PLAYING:
            self.tick_playing()
        return

    def tick_playing( self ):
        self.player.tick()
        for opponent in self.opponents:
            opponent.tick()

    def draw( self ):
        if self.game_mode in [GAME_MODE_MAIN_MENU,GAME_MODE_HELP]:
            self.draw_menu()
        if self.game_mode == GAME_MODE_PLAYING:
            self.draw_playing()
        pygame.display.flip()
        return

    def draw_menu( self ):
        if self.game_mode == GAME_MODE_MAIN_MENU:
            self.screen.blit( gfx[GFX_MAIN_MENU], (0,0) )
        if self.game_mode == GAME_MODE_HELP:
            self.screen.blit( gfx[GFX_HELP_SCREEN], (0,0) )
        for button in self.buttons:
            button.draw( self.screen )
        return

    def draw_playing( self ):
        self.screen.blit( self.garden, (0,0) )
        self.ai.draw_dwarfs( self.screen )
        self.ai.draw_flowers( self.screen )
        self.player.draw( self.screen )
        for opponent in self.opponents:
            opponent.draw( self.screen )
        for button in self.buttons:
            button.draw( self.screen )
        return

    def start_game_loop( self ):
        while self.run_game:
            ticks_start = pygame.time.get_ticks()

            # handle events
            self.handle_events()
            # update objects
            self.tick()
            # draw stuff
            self.draw()

            # wait a little to get 30fps
            ticks_end = pygame.time.get_ticks()
            ticks_diff = ticks_end - ticks_start
            if ticks_diff < 1000/FRAMES_PER_SECOND:
                pygame.time.delay( 1000/int(FRAMES_PER_SECOND)-ticks_diff )
        return

if __name__ == '__main__':
    game = Game()
    game.start()