#!/usr/bin/env python

import pygame

AREA_STARTED_AT_UPPER_LEFT = 0
AREA_STARTED_AT_LOWER_LEFT = 1
AREA_STARTED_AT_UPPER_RIGHT = 2
AREA_STARTED_AT_LOWER_RIGHT = 3

FREERGBA = (0,0,0,0)
AREA_COLOR = (0,0,255,50)

class AreaIdGenerator:
    def __init__( self ):
        self.id = 0
        return

    def new_id( self ):
        self.id = self.id + 1
        return self.id

pygame.init()
areaId = AreaIdGenerator()

class AreaOFMovement:
    def __init__( self, left, top, width, height ):
        self.id = areaId.new_id()
        self.top = top
        self.left = left
        self.width = width
        self.height = height
        return

    def __str__( self ):
        s = "AreaOfMovement %i from (%i,%i) to (%i,%i)" % (self.id,self.left,self.top,self.left+self.width,self.top+self.height)
        return s

    def draw( self, surface ):
        pygame.draw.rect( surface, AREA_COLOR, (self.left,self.top,self.width,self.height) )
        return

    def collides_with( self, other ):
        rect1 = pygame.rect.Rect( self.left, self.top, self.width, self.height )
        rect2 = pygame.rect.Rect( other.left, other.top, other.width, other.height )
        return rect1.colliderect( rect2 )

    def create_neighbours( self, surface, freeRGBA ):
        neighbours = []
        for neighbour in self.create_top_neighbours(surface,freeRGBA):
            neighbours.append( neighbour )
        for neighbour in self.create_bottom_neighbours(surface,freeRGBA):
            neighbours.append( neighbour )
        for neighbour in self.create_left_neighbours(surface,freeRGBA):
            neighbours.append( neighbour )
        for neighbour in self.create_right_neighbours(surface,freeRGBA):
            neighbours.append( neighbour )
        return neighbours

    def create_top_neighbours( self, surface, freeRGBA ):
        neighbours = []
        if self.top <= 0:
            return []
        start_x = None
        end_x = None
        for x in range(self.left,self.left+self.width+1):

            if surface.get_at((x,self.top-1)) == freeRGBA:
                if start_x is None:
                    start_x = x
                end_x = x
            else:
                if start_x != None and end_x != None:
                    # found neighbour
                    neighbours.append( create_area_from_lower_to_upper(surface,start_x,end_x,self.top-1,freeRGBA) )
                    start_x = None
                    end_x = None

        if start_x != None and end_x != None:
            # found last neighbour
            neighbours.append( create_area_from_lower_to_upper(surface,start_x,end_x,self.top-1,freeRGBA) )

        return neighbours

    def create_bottom_neighbours( self, surface, freeRGBA ):
        neighbours = []
        if self.top+self.height >= surface.get_height()-1:
            return neighbours
        start_x = None
        end_x = None
        for x in range(self.left,self.left+self.width+1):
            if surface.get_at((x,self.top+self.height+1)) == freeRGBA:
                if start_x is None:
                    start_x = x
                end_x = x
            else:
                if start_x != None and end_x != None:
                    # found neighbour
                    neighbours.append( create_area_from_upper_to_lower(surface,start_x,end_x,self.top+self.height+1,freeRGBA) )
                    start_x = None
                    end_x = None
        if start_x != None and end_x != None:
            # found last neighbour
            neighbours.append( create_area_from_upper_to_lower(surface,start_x,end_x,self.top+self.height+1,freeRGBA) )
        return neighbours

    def create_left_neighbours( self, surface, freeRGBA ):
        neighbours = []
        if self.left<=0:
            return neighbours
        start_y = None
        end_y = None
        for y in range(self.top,self.top+self.height+1):
            if surface.get_at((self.left-1,y)) == freeRGBA:
                if start_y == None:
                    start_y = y
                end_y = y
            else:
                if start_y != None and end_y != None:
                    neighbours.append( create_area_from_right_to_left(surface,self.left-1,start_y,end_y,freeRGBA) )
                    start_y = None
                    end_y = None
        if start_y != None and end_y != None:
            # found last neighbour
            neighbours.append( create_area_from_right_to_left(surface,self.left-1,start_y,end_y,freeRGBA) )
        return neighbours

    def create_right_neighbours( self, surface, freeRGBA ):
        neighbours = []
        if self.left+self.width >= surface.get_width()-1:
            return neighbours
        start_y = None
        end_y = None
        for y in range(self.top,self.top+self.height+1):
            if surface.get_at((self.left+self.width+1,y)) == freeRGBA:
                if start_y == None:
                    start_y = y
                end_y = y
            else:
                if start_y != None and end_y != None:
                    neighbours.append( create_area_from_left_to_right(surface,self.left+self.width+1,start_y,end_y,freeRGBA) )
                    start_y = None
                    end_y = None
        if start_y != None and end_y != None:
            # found last neighbour
            neighbours.append( create_area_from_left_to_right(surface,self.left+self.width+1,start_y,end_y,freeRGBA) )
        return neighbours


class AreasOFMovement:
    def __init__( self, surface, freeRGBA ):
        self.surface = surface
        self.freeRGBA = freeRGBA
        self.areas = []
        self.evaluate_areas = []
        return

    def calculate( self ):
        self.areas = []
        self.evaluate_areas = []

        # get the upper left of the first area
        start = get_free_upper_left( self.surface, (0,0), self.freeRGBA )
        print "free_uper_left =", start
        if start == (0,0):
            return

        # create the first area
        firstArea = create_area_from_upper_left( self.surface, start, self.freeRGBA )
        self.areas.append( firstArea )
        index = 0
        while index<=20 and index<self.areas.__len__():
            print index, self.areas.__len__()
            area = self.areas[index]
            for newArea in self.areas[index].create_neighbours( self.surface, self.freeRGBA ):
                isNewArea = True
                for existingArea in self.areas:
                    if newArea.collides_with( existingArea ):
                        isNewArea = False
                if isNewArea:
                    self.areas.append( newArea )
            index = index + 1

        # append neighbour areas
        for neighbour in firstArea.create_neighbours( self.surface, self.freeRGBA ):
            self.areas.append( neighbour )

        return

    def print_description( self ):
        for area in self.areas:
            print area


def get_free_upper_left( surface, start, freeRGBA ):
    for x in range(start[0],surface.get_width()):
        start_y = start[1] if x==start[0] else 0
        for y in range(start_y,surface.get_height()):
            print "Position", (x,y), " pixel =", surface.get_at((x,y))
            if surface.get_at((x,y)) == freeRGBA:
                return (x,y)
    return (-1,-1)

def create_area_from_lower_to_upper( surface, left, right, bottom, freeRGBA ):
    area_valid = True
    top = bottom
    while top>=0 and area_valid:
        if is_free_horizontal_line( surface, left, right, top-1, freeRGBA ):
            top = top - 1
        else:
            area_valid = False
    return AreaOFMovement(left,top,right-left,bottom-top)

def create_area_from_upper_to_lower( surface, left, right, top, freeRGBA ):
    area_valid = True
    bottom = top
    while bottom<surface.get_height() and area_valid:
        if is_free_horizontal_line( surface, left, right, bottom+1, freeRGBA ):
            bottom = bottom + 1
        else:
            area_valid = False
    return AreaOFMovement(left,top,right-left,bottom-top)

def create_area_from_left_to_right( surface, left, top, bottom, freeRGBA ):
    area_valid = True
    right = left
    while right<surface.get_height() and area_valid:
        if is_free_vertical_line( surface, right+1, top, bottom, freeRGBA ):
            right = right + 1
        else:
            area_valid = False
    return AreaOFMovement(left,top,right-left,bottom-top)

def create_area_from_right_to_left( surface, right, top, bottom, freeRGBA ):
    area_valid = True
    left = right
    while left>=0 and area_valid:
        if is_free_vertical_line( surface, left-1, top, bottom, freeRGBA ):
            left = left - 1
        else:
            area_valid = False
    return AreaOFMovement(left,top,right-left,bottom-top)

def create_area_from_upper_left( surface, start, freeRGBA ):
    error = None

    if start[0]<0 or start[1]<0:
        return error

    x = start[0]
    y = start[1]
    increase_x = True
    increase_y = True

    while True:
        new_x = x
        new_y = y

        if increase_x:
            if is_free_rect( surface, start, (x+1,y), freeRGBA ):
                new_x = x + 1
            else:
                increase_x = False

        if increase_y:
            if is_free_rect( surface, start, (x,y+1), freeRGBA ):
                new_y = y + 1
            else:
                increase_y = False

        if increase_x==False and increase_x==False:
            return AreaOFMovement( start[0], start[1], x-start[0], y-start[1] )

        x = new_x
        y = new_y

        if x >= surface.get_width() or y >= surface.get_height():
            return error

    return error






def raise_max_rect_from_lower_left( surface, start, freeRGBA ):
    error = (-1,-1)

    if start[0]<0 or start[1]>=surface.get_height():
        return error

    x = start[0]
    y = start[1]
    increase_x = True
    decrease_y = True

    while True:
        new_x = x
        new_y = y

        if increase_x:
            if is_free_rect( surface, start, (x+1,y), freeRGBA ):
                new_x = x + 1
            else:
                increase_x = False

        if decrease_y:
            if is_free_rect( surface, start, (x,y-1), freeRGBA ):
                new_y = y - 1
            else:
                decrease_y = False

        if increase_x==False and decrease_x==False:
            return (x,y)

        x = new_x
        y = new_y

        if x >= surface.get_width() or y < 0:
            return error

def raise_max_rect_from_upper_right( surface, start, freeRGBA ):
    error = (-1,-1)

    if start[0]>=surface.get_width() or start[1]<0:
        return error

    x = start[0]
    y = start[1]
    decrease_x = True
    increase_y = True

    while True:
        new_x = x
        new_y = y

        if decrease_x:
            if is_free_rect( surface, start, (x-1,y), freeRGBA ):
                new_x = x - 1
            else:
                decrease_x = False

        if increase_y:
            if is_free_rect( surface, start, (x,y+1), freeRGBA ):
                new_y = y + 1
            else:
                increase_y = False

        if decrease_x==False and increase_x==False:
            return (x,y)

        x = new_x
        y = new_y

        if x < 0 or y >= surface.get_height():
            return error

def raise_max_rect_from_lower_right( surface, start, freeRGBA ):
    error = (-1,-1)

    if start[0]>=surface.get_width() or start[1]>=surface.get_height():
        return error

    x = start[0]
    y = start[1]
    decrease_x = True
    decrease_y = True

    while True:
        new_x = x
        new_y = y

        if decrease_x:
            if is_free_rect( surface, start, (x-1,y), freeRGBA ):
                new_x = x - 1
            else:
                decrease_x = False

        if decrease_y:
            if is_free_rect( surface, start, (x,y-1), freeRGBA ):
                new_y = y - 1
            else:
                decrease_y = False

        if decrease_x==False and decrease_x==False:
            return (x,y)

        x = new_x
        y = new_y

        if x < 0 or y < 0:
            return error

def is_free_rect( surface, p1, p2, freeRGBA ):
    upper_left  = ( min(p1[0],p2[0]), min(p1[1],p2[1]) )
    if upper_left[0] < 0 or upper_left[1] < 0:
        return False

    lower_right = ( max(p1[0],p2[0]), max(p1[1],p2[1]) )
    if lower_right[0] >= surface.get_height() or lower_right[1] >= surface.get_height():
        return False

    for y in range(upper_left[1],lower_right[1]+1):
        if is_free_horizontal_line( surface, upper_left[0], lower_right[0], y, freeRGBA ) == False:
            return False
    return True

def is_free_horizontal_line( surface, left, right, y, freeRGBA ):
    if left<0 or left>=surface.get_width() or right<0 or right>=surface.get_width() or right<left or y<0 or y>=surface.get_height():
        return False
    for x in range(left,right+1):
        if surface.get_at((x,y)) != freeRGBA:
            return False
    return True

def is_free_vertical_line( surface, x, top, bottom, freeRGBA ):
    if x<0 or x>=surface.get_width() or top<0 or top>=surface.get_width() or bottom<0 or bottom>=surface.get_height() or top>bottom:
        return False
    for y in range(top,bottom+1):
        if surface.get_at((x,y)) != freeRGBA:
            return False
    return True

def test():
    screen = pygame.display.set_mode( (800,600) )
    #surface = pygame.image.load( "test.png" ).convert_alpha()
    surface = pygame.image.load( "level01_metadata.png" ).convert_alpha()

    aofm = AreasOFMovement( surface, FREERGBA )
    aofm.calculate()
    aofm.print_description()

    run_test = True
    while run_test:
        # handle events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run_test = False
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                run_test = False
        # draw stuff
        screen.blit( surface, (0,0) )
        for area in aofm.areas:
            area.draw( surface )
        pygame.display.flip()

    pygame.quit()


if __name__ == '__main__':
    test()
